import {
  Directive,
  ElementRef,
  EventEmitter,
  HostListener,
  Output,
  Input,
  AfterViewInit,
  NgZone
} from '@angular/core';

import { FileUploadItem } from '@ucap/api';

import { FileUploadQueueComponent } from '../components/file-upload-queue.component';

@Directive({
  selector: 'input[ucapFileUploadFor], div[ucapFileUploadFor]'
})
export class FileUploadForDirective implements AfterViewInit {
  @Input()
  fileUploadQueue: FileUploadQueueComponent;

  @Input()
  validator: (fileList: FileList) => Promise<boolean>;

  @Output()
  public fileDragEnter = new EventEmitter<DataTransferItemList>();

  @Output()
  public fileDragOver = new EventEmitter<void>();

  @Output()
  public fileDragLeave = new EventEmitter<void>();

  @Output()
  public fileSelected = new EventEmitter<FileUploadItem[]>();

  dragOver = false;

  constructor(
    private elementRef: ElementRef,
    private readonly ngZone: NgZone
  ) {}

  ngAfterViewInit(): void {}

  @HostListener('window:dragenter', ['$event'])
  public onDragEnter(event: DragEvent): any {
    if (!this.isFileDrag(event.dataTransfer)) {
      return;
    }

    if (!this.dragOver) {
      this.fileDragEnter.emit(event.dataTransfer.items);
      this.dragOver = true;
      if (!!this.fileUploadQueue) {
        this.fileUploadQueue.onDragEnter(event.dataTransfer.items);
      }
    }
  }

  @HostListener('window:dragover', ['$event'])
  public onDragOver(event: DragEvent): any {
    if (!this.isFileDrag(event.dataTransfer)) {
      return;
    }

    // if (this.fileUploadQueue.isEventInElement(event)) {
    //   event.dataTransfer.dropEffect = 'copy';
    // } else {
    //   event.dataTransfer.dropEffect = 'none';
    // }
    event.preventDefault();
  }

  @HostListener('window:dragleave', ['$event'])
  public onDragLeave(event: DragEvent): any {
    if (!this.isFileDrag(event.dataTransfer)) {
      return;
    }

    if (event && event.pageX === 0 && event.pageY === 0) {
      this.fileDragLeave.emit();
      this.dragOver = false;
      if (!!this.fileUploadQueue) {
        this.fileUploadQueue.onDragLeave();
      }
    }
  }

  @HostListener('change')
  public onChange(): any {
    const files = this.elementRef.nativeElement.files;
    this.fileSelected.emit(FileUploadItem.fromFiles(files));
    this.elementRef.nativeElement.value = '';
  }

  @HostListener('window:drop', ['$event'])
  public onDrop(event: any): any {
    if (!this.isFileDrag(event.dataTransfer)) {
      return;
    }

    const self = this;
    const files: FileList = event.dataTransfer.files;

    event.preventDefault();
    event.stopPropagation();
    this.elementRef.nativeElement.value = '';
    this.dragOver = false;

    if (!!this.validator) {
      this.validator(files)
        .then(async result => {
          if (!result) {
            if (!!this.fileUploadQueue) {
              this.fileUploadQueue.onUploadComplete();
            }
            return;
          } else {
            const fileUploadItems = FileUploadItem.fromFiles(files);
            self.fileSelected.emit(fileUploadItems);

            if (!!self.fileUploadQueue) {
              self.fileUploadQueue.onDrop(fileUploadItems);
            }
          }
        })
        .catch(err => {
          if (!!this.fileUploadQueue) {
            this.fileUploadQueue.onUploadComplete();
          }
        });
    }
  }

  private isFileDrag(dataTransfer: DataTransfer): boolean {
    if (0 >= dataTransfer.items.length) {
      return false;
    }

    // tslint:disable-next-line: prefer-for-of
    for (let i = 0; i < dataTransfer.items.length; i++) {
      const element = dataTransfer.items[i];
      if ('file' !== element.kind) {
        return false;
      }
    }

    return true;
  }
}
